'use strict';

var __metamagical_withMeta = function metamagical_withMeta(object, meta) {
  var parent = Object.getPrototypeOf(object);var oldMeta = object[Symbol.for('@@meta:magical')] || {};if (parent && parent[Symbol.for('@@meta:magical')] === oldMeta) {
    oldMeta = {};
  }Object.keys(meta).forEach(function (key) {
    if (/^~/.test(key)) {
      oldMeta[key.slice(1)] = meta[key];
    } else {
      oldMeta[key] = meta[key];
    }
  });object[Symbol.for('@@meta:magical')] = oldMeta;return object;
},
    _Refinable$refine;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

//---------------------------------------------------------------------
//
// This source file is part of the Meta:Magical project.
//
// See LICENCE for licence information.
// See CONTRIBUTORS for the list of contributors to the project.
//
//---------------------------------------------------------------------

// --[ Dependencies ]--------------------------------------------------
var Refinable = require('refinable');
var Maybe = require('data.maybe');
var Stability = require('./stability');

// --[ Helpers ]-------------------------------------------------------
function required(object, name) {
  throw new Error('This ' + object + ' does not define ' + name + '.\n\nYou must refine this object and define the missing property before\nyou can use it. For example:\n\n    const newObject = ' + object + '.refine({ ' + name + ': ... });\n  ');
}

function unique(xs) {
  return [].concat(_toConsumableArray(new Set(xs)));
}

function flatten(xss) {
  return xss.reduce(function (l, r) {
    return l.concat(r);
  }, []);
}

// --[ Base for Fields ]-----------------------------------------------

/*~
 * ---
 * module: metamagical-interface/lib/fields
 */
var Field = __metamagical_withMeta(Refinable.refine((_Refinable$refine = {
  get name() {
    required('Field', 'name');
  },

  get allowInheritance() {
    return false;
  },

  get allowPropagation() {
    return false;
  },

  merge: function merge(values) {
    return Maybe.Just(values);
  }
}, __metamagical_withMeta(Object.getOwnPropertyDescriptor(_Refinable$refine, 'name').get, {
  'name': 'name',
  'source': 'get name() {\n    required(\'Field\', \'name\');\n  }',
  'signature': 'get name()',
  'belongsTo': function belongsTo() {
    return Field;
  },
  'documentation': '\n   '
}), __metamagical_withMeta(Object.getOwnPropertyDescriptor(_Refinable$refine, 'allowInheritance').get, {
  'name': 'allowInheritance',
  'source': 'get allowInheritance() {\n    return false;\n  }',
  'signature': 'get allowInheritance()',
  'belongsTo': function belongsTo() {
    return Field;
  },
  'documentation': '\n   '
}), __metamagical_withMeta(Object.getOwnPropertyDescriptor(_Refinable$refine, 'allowPropagation').get, {
  'name': 'allowPropagation',
  'source': 'get allowPropagation() {\n    return false;\n  }',
  'signature': 'get allowPropagation()',
  'belongsTo': function belongsTo() {
    return Field;
  },
  'documentation': '\n   '
}), __metamagical_withMeta(_Refinable$refine['merge'], {
  'name': 'merge',
  'source': 'merge(values) {\n    return Maybe.Just(values);\n  }',
  'signature': 'merge(values)',
  'belongsTo': function belongsTo() {
    return Field;
  },
  'documentation': '\n   '
}), _Refinable$refine)), {
  'name': 'Field',
  'source': 'Refinable.refine({\n  /*~\n   */\n  get name() {\n    required(\'Field\', \'name\');\n  },\n\n  /*~\n   */\n  get allowInheritance() {\n    return false;\n  },\n\n  /*~\n   */\n  get allowPropagation() {\n    return false;\n  },\n\n  /*~\n   */\n  merge(values) {\n    return Maybe.Just(values);\n  }\n})',
  'location': {
    'filename': 'packages/interface/src/fields.js',
    'start': {
      'line': 42,
      'column': 0
    },
    'end': {
      'line': 66,
      'column': 3
    }
  },
  'module': 'metamagical-interface/lib/fields',
  'homepage': 'https://github.com/origamitower/metamagical#readme',
  'licence': 'MIT',
  'authors': ['Quildreen Motta <queen@robotlolita.me>'],
  'repository': 'git://github.com/origamitower/metamagical.git',
  'npmPackage': 'metamagical-interface',
  'copyright': '(c) 2016 Quildreen Motta',
  'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
  'documentation': ''
});

// --[ Built-in Fields ]-----------------------------------------------

/*~
 * ---
 * module   : metamagical-interface/lib/fields
 * isModule : true
 */
module.exports = __metamagical_withMeta({
  Field: Field,

  // ---[ Social Metadata ]--------------------------------------------
  copyright: Field.refine({
    name: 'copyright',
    allowInheritance: true,
    allowPropagation: true,
    merge: function merge(values) {
      return Maybe.Just(unique(values));
    }
  }),

  authors: Field.refine({
    name: 'authors',
    allowInheritance: true,
    allowPropagation: true,
    merge: function merge(values) {
      return Maybe.Just(unique(flatten(values)));
    }
  }),

  maintainers: Field.refine({
    name: 'maintainers',
    allowInheritance: true,
    allowPropagation: true,
    merge: function merge(values) {
      return Maybe.Just(unique(flatten(values)));
    }
  }),

  licence: Field.refine({
    name: 'licence',
    allowInheritance: true,
    allowPropagation: true,
    merge: function merge(values) {
      return Maybe.Just(unique(values));
    }
  }),

  since: Field.refine({
    name: 'since',
    allowInheritance: true
  }),

  deprecated: Field.refine({
    name: 'deprecated',
    allowInheritance: true
  }),

  repository: Field.refine({
    name: 'repository',
    allowInheritance: true
  }),

  stability: Field.refine({
    name: 'stability',
    allowInheritance: true,
    allowPropagation: true,
    merge: function merge(values) {
      if (values.length === 0) {
        return Maybe.Nothing();
      } else {
        return Maybe.Just(values.reduce(function (l, r) {
          var left = Stability.fromIdentifier(l).index;
          var right = Stability.fromIdentifier(r).index;

          return left < right ? l : /* else */r;
        }));
      }
    }
  }),

  homepage: Field.refine({
    name: 'homepage',
    allowInheritance: true
  }),

  // ---[ Organisational Metadata ]------------------------------------
  category: Field.refine({
    name: 'category'
  }),

  tags: Field.refine({
    name: 'tags'
  }),

  // ---[ Definition Metadata ]----------------------------------------
  npmPackage: Field.refine({
    name: 'npmPackage',
    allowInheritance: true
  }),

  module: Field.refine({
    name: 'module',
    allowInheritance: true
  }),

  isModule: Field.refine({
    name: 'isModule'
  }),

  belongsTo: Field.refine({
    name: 'belongsTo'
  }),

  name: Field.refine({
    name: 'name'
  }),

  location: Field.refine({
    name: 'location'
  }),

  source: Field.refine({
    name: 'source'
  }),

  // ---[ Usage Metadata ]---------------------------------------------
  documentation: Field.refine({
    name: 'documentation'
  }),

  examples: Field.refine({
    name: 'examples'
  }),

  signature: Field.refine({
    name: 'signature'
  }),

  type: Field.refine({
    name: 'type'
  }),

  throws: Field.refine({
    name: 'throws'
  }),

  parameters: Field.refine({
    name: 'parameters'
  }),

  returns: Field.refine({
    name: 'returns'
  }),

  complexity: Field.refine({
    name: 'complexity'
  }),

  // ---[ Execution Metadata ]-----------------------------------------
  platforms: Field.refine({
    name: 'platforms',
    allowInheritance: true,
    allowPropagation: true,
    merge: function merge(values) {
      return Maybe.Just(unique(flatten(values)));
    }
  }),

  portable: Field.refine({
    name: 'portable',
    allowInheritance: true,
    allowPropagation: true,
    merge: function merge(values) {
      if (values.length === 0) {
        return Maybe.Nothing();
      } else {
        return Maybe.Just(values.every(Boolean));
      }
    }
  })

}, {
  'name': 'exports',
  'source': '{\n  Field: Field,\n\n  // ---[ Social Metadata ]--------------------------------------------\n  copyright: Field.refine({\n    name: \'copyright\',\n    allowInheritance: true,\n    allowPropagation: true,\n    merge(values) {\n      return Maybe.Just(unique(values));\n    }\n  }),\n\n  authors: Field.refine({\n    name: \'authors\',\n    allowInheritance: true,\n    allowPropagation: true,\n    merge(values) {\n      return Maybe.Just(unique(flatten(values)));\n    }\n  }),\n\n  maintainers: Field.refine({\n    name: \'maintainers\',\n    allowInheritance: true,\n    allowPropagation: true,\n    merge(values) {\n      return Maybe.Just(unique(flatten(values)));\n    }\n  }),\n\n  licence: Field.refine({\n    name: \'licence\',\n    allowInheritance: true,\n    allowPropagation: true,\n    merge(values) {\n      return Maybe.Just(unique(values));\n    }\n  }),\n\n  since: Field.refine({\n    name: \'since\',\n    allowInheritance: true\n  }),\n\n  deprecated: Field.refine({\n    name: \'deprecated\',\n    allowInheritance: true\n  }),\n\n  repository: Field.refine({\n    name: \'repository\',\n    allowInheritance: true\n  }),\n\n  stability: Field.refine({\n    name: \'stability\',\n    allowInheritance: true,\n    allowPropagation: true,\n    merge(values) {\n      if (values.length === 0) {\n        return Maybe.Nothing();\n      } else {\n        return Maybe.Just(values.reduce((l, r) => {\n          const left  = Stability.fromIdentifier(l).index;\n          const right = Stability.fromIdentifier(r).index;\n\n          return left < right ?  l\n          :      /* else */      r;\n        }));\n      }\n    }\n  }),\n\n  homepage: Field.refine({\n    name: \'homepage\',\n    allowInheritance: true\n  }),\n\n\n  // ---[ Organisational Metadata ]------------------------------------\n  category: Field.refine({\n    name: \'category\'\n  }),\n\n  tags: Field.refine({\n    name: \'tags\'\n  }),\n\n\n  // ---[ Definition Metadata ]----------------------------------------\n  npmPackage: Field.refine({\n    name: \'npmPackage\',\n    allowInheritance: true\n  }),\n\n  module: Field.refine({\n    name: \'module\',\n    allowInheritance: true\n  }),\n\n  isModule: Field.refine({\n    name: \'isModule\'\n  }),\n\n  belongsTo: Field.refine({\n    name: \'belongsTo\'\n  }),\n\n  name: Field.refine({\n    name: \'name\'\n  }),\n\n  location: Field.refine({\n    name: \'location\'\n  }),\n\n  source: Field.refine({\n    name: \'source\'\n  }),\n\n\n  // ---[ Usage Metadata ]---------------------------------------------\n  documentation: Field.refine({\n    name: \'documentation\'\n  }),\n\n  examples: Field.refine({\n    name: \'examples\'\n  }),\n\n  signature: Field.refine({\n    name: \'signature\'\n  }),\n\n  type: Field.refine({\n    name: \'type\'\n  }),\n\n  throws: Field.refine({\n    name: \'throws\'\n  }),\n\n  parameters: Field.refine({\n    name: \'parameters\'\n  }),\n\n  returns: Field.refine({\n    name: \'returns\'\n  }),\n\n  complexity: Field.refine({\n    name: \'complexity\'\n  }),\n\n\n  // ---[ Execution Metadata ]-----------------------------------------\n  platforms: Field.refine({\n    name: \'platforms\',\n    allowInheritance: true,\n    allowPropagation: true,\n    merge(values) {\n      return Maybe.Just(unique(flatten(values)));\n    }\n  }),\n\n  portable: Field.refine({\n    name: \'portable\',\n    allowInheritance: true,\n    allowPropagation: true,\n    merge(values) {\n      if (values.length === 0) {\n        return Maybe.Nothing();\n      } else {\n        return Maybe.Just(values.every(Boolean));\n      }\n    }\n  })\n\n}',
  'location': {
    'filename': 'packages/interface/src/fields.js',
    'start': {
      'line': 76,
      'column': 0
    },
    'end': {
      'line': 255,
      'column': 2
    }
  },
  'module': 'metamagical-interface/lib/fields',
  'homepage': 'https://github.com/origamitower/metamagical#readme',
  'licence': 'MIT',
  'authors': ['Quildreen Motta <queen@robotlolita.me>'],
  'repository': 'git://github.com/origamitower/metamagical.git',
  'npmPackage': 'metamagical-interface',
  'copyright': '(c) 2016 Quildreen Motta',
  'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
  'isModule': true,
  'documentation': ''
});