'use strict';

var __metamagical_withMeta = function metamagical_withMeta(object, meta) {
  var parent = Object.getPrototypeOf(object);var oldMeta = object[Symbol.for('@@meta:magical')] || {};if (parent && parent[Symbol.for('@@meta:magical')] === oldMeta) {
    oldMeta = {};
  }Object.keys(meta).forEach(function (key) {
    if (/^~/.test(key)) {
      oldMeta[key.slice(1)] = meta[key];
    } else {
      oldMeta[key] = meta[key];
    }
  });object[Symbol.for('@@meta:magical')] = oldMeta;return object;
},
    _Refinable$refine,
    _index;

//---------------------------------------------------------------------
//
// This source file is part of the Meta:Magical project.
//
// See LICENCE for licence information.
// See CONTRIBUTORS for the list of contributors to the project.
//
//---------------------------------------------------------------------

// --[ Dependencies ]--------------------------------------------------
var Refinable = require('refinable');

// --[ Implementation ]------------------------------------------------

/*~
 * Handles describing and normalising stability identifiers.
 *
 * ---
 * name        : module stability
 * module      : metamagical-interface/lib/stability
 * isModule    : true
 * category    : Metadata
 * portability : portable
 * platforms:
 *   - ECMAScript 5
 *   - ECMAScript 3 (with `es5-shim`)
 */
module.exports = __metamagical_withMeta(Refinable.refine((_Refinable$refine = {
  fromIdentifier: function fromIdentifier(id) {
    if (this.index.hasOwnProperty(id)) {
      return this.index[id];
    } else {
      throw new Error('No stability with id "' + id + '"');
    }
  },
  index: (_index = {
    deprecated: Refinable.refine({
      index: 0,
      name: 'Deprecated',
      description: '\nThis feature is known to be problematic, and will either be entirely\nremoved from the system, or completely redesigned. You should not rely\non it.'
    }),
    experimental: Refinable.refine({
      index: 1,
      name: 'Experimental',
      description: '\nThis feature is experimental and likely to change (or be removed) in the\nfuture.'
    }),
    stable: Refinable.refine({
      index: 2,
      name: 'Stable',
      description: '\nThis feature is stable, and its API is unlikely to change (unless deemed\nnecessary for security or other important reasons). You should expect\nbackwards compatibility with the system, and a well-defined and automated\n(if possible) migration path if it changes.'
    }),
    locked: Refinable.refine({
      index: 3,
      name: 'Locked',
      description: '\nThis API will not change, however security and other bug fixes will still\nbe applied.'
    })
  }, __metamagical_withMeta(_index['deprecated'], {
    'name': 'deprecated',
    'source': 'Refinable.refine({\n      index: 0,\n      name:  \'Deprecated\',\n      description: `\nThis feature is known to be problematic, and will either be entirely\nremoved from the system, or completely redesigned. You should not rely\non it.`\n    })',
    'location': {
      'filename': 'packages/interface/src/stability.js',
      'start': {
        'line': 58,
        'column': 9
      },
      'end': {
        'line': 113,
        'column': 3
      }
    },
    'module': 'metamagical-interface/src/stability',
    'homepage': 'https://github.com/origamitower/metamagical#readme',
    'licence': 'MIT',
    'authors': ['Quildreen Motta <queen@robotlolita.me>'],
    'repository': 'git://github.com/origamitower/metamagical.git',
    'npmPackage': 'metamagical-interface',
    'copyright': '(c) 2016 Quildreen Motta',
    'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
    'stability': 'stable',
    'category': 'Stability entry',
    'documentation': ''
  }), __metamagical_withMeta(_index['experimental'], {
    'name': 'experimental',
    'source': 'Refinable.refine({\n      index: 1,\n      name:  \'Experimental\',\n      description: `\nThis feature is experimental and likely to change (or be removed) in the\nfuture.`\n    })',
    'location': {
      'filename': 'packages/interface/src/stability.js',
      'start': {
        'line': 58,
        'column': 9
      },
      'end': {
        'line': 113,
        'column': 3
      }
    },
    'module': 'metamagical-interface/src/stability',
    'homepage': 'https://github.com/origamitower/metamagical#readme',
    'licence': 'MIT',
    'authors': ['Quildreen Motta <queen@robotlolita.me>'],
    'repository': 'git://github.com/origamitower/metamagical.git',
    'npmPackage': 'metamagical-interface',
    'copyright': '(c) 2016 Quildreen Motta',
    'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
    'stability': 'stable',
    'category': 'Stability entry',
    'documentation': ''
  }), __metamagical_withMeta(_index['stable'], {
    'name': 'stable',
    'source': 'Refinable.refine({\n      index: 2,\n      name:  \'Stable\',\n      description: `\nThis feature is stable, and its API is unlikely to change (unless deemed\nnecessary for security or other important reasons). You should expect\nbackwards compatibility with the system, and a well-defined and automated\n(if possible) migration path if it changes.`\n    })',
    'location': {
      'filename': 'packages/interface/src/stability.js',
      'start': {
        'line': 58,
        'column': 9
      },
      'end': {
        'line': 113,
        'column': 3
      }
    },
    'module': 'metamagical-interface/src/stability',
    'homepage': 'https://github.com/origamitower/metamagical#readme',
    'licence': 'MIT',
    'authors': ['Quildreen Motta <queen@robotlolita.me>'],
    'repository': 'git://github.com/origamitower/metamagical.git',
    'npmPackage': 'metamagical-interface',
    'copyright': '(c) 2016 Quildreen Motta',
    'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
    'stability': 'stable',
    'category': 'Stability entry',
    'documentation': ''
  }), __metamagical_withMeta(_index['locked'], {
    'name': 'locked',
    'source': 'Refinable.refine({\n      index: 3,\n      name:  \'Locked\',\n      description: `\nThis API will not change, however security and other bug fixes will still\nbe applied.`\n    })',
    'location': {
      'filename': 'packages/interface/src/stability.js',
      'start': {
        'line': 58,
        'column': 9
      },
      'end': {
        'line': 113,
        'column': 3
      }
    },
    'module': 'metamagical-interface/src/stability',
    'homepage': 'https://github.com/origamitower/metamagical#readme',
    'licence': 'MIT',
    'authors': ['Quildreen Motta <queen@robotlolita.me>'],
    'repository': 'git://github.com/origamitower/metamagical.git',
    'npmPackage': 'metamagical-interface',
    'copyright': '(c) 2016 Quildreen Motta',
    'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
    'stability': 'stable',
    'category': 'Stability entry',
    'documentation': ''
  }), _index)
}, __metamagical_withMeta(_Refinable$refine['fromIdentifier'], {
  'name': 'fromIdentifier',
  'source': 'fromIdentifier(id) {\n    if (this.index.hasOwnProperty(id)) {\n      return this.index[id];\n    } else {\n      throw new Error(`No stability with id "${id}"`);\n    }\n  }',
  'signature': 'fromIdentifier(id)',
  'location': {
    'filename': 'packages/interface/src/stability.js',
    'start': {
      'line': 30,
      'column': 34
    },
    'end': {
      'line': 114,
      'column': 1
    }
  },
  'module': 'metamagical-interface/src/stability',
  'homepage': 'https://github.com/origamitower/metamagical#readme',
  'licence': 'MIT',
  'authors': ['Quildreen Motta <queen@robotlolita.me>'],
  'repository': 'git://github.com/origamitower/metamagical.git',
  'npmPackage': 'metamagical-interface',
  'copyright': '(c) 2016 Quildreen Motta',
  'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
  'category': 'Constructing stability entries',
  'stability': 'stable',
  'type': 'Stability.(String) => StabilityEntry\n \n',
  'documentation': '\nConverts a textual identifier of stability to a structured\nrepresentation of the stability.\n'
}), __metamagical_withMeta(_Refinable$refine['index'], {
  'name': 'index',
  'source': '{\n    /*~\n     * ---\n     * stability : stable\n     * category  : Stability entry\n     */\n    deprecated: Refinable.refine({\n      index: 0,\n      name:  \'Deprecated\',\n      description: `\nThis feature is known to be problematic, and will either be entirely\nremoved from the system, or completely redesigned. You should not rely\non it.`\n    }),\n\n    /*~\n     * ---\n     * stability : stable\n     * category  : Stability entry\n     */\n    experimental: Refinable.refine({\n      index: 1,\n      name:  \'Experimental\',\n      description: `\nThis feature is experimental and likely to change (or be removed) in the\nfuture.`\n    }),\n\n    /*~\n     * ---\n     * stability : stable\n     * category  : Stability entry\n     */\n    stable: Refinable.refine({\n      index: 2,\n      name:  \'Stable\',\n      description: `\nThis feature is stable, and its API is unlikely to change (unless deemed\nnecessary for security or other important reasons). You should expect\nbackwards compatibility with the system, and a well-defined and automated\n(if possible) migration path if it changes.`\n    }),\n\n    /*~\n     * ---\n     * stability : stable\n     * category  : Stability entry\n     */\n    locked: Refinable.refine({\n      index: 3,\n      name:  \'Locked\',\n      description: `\nThis API will not change, however security and other bug fixes will still\nbe applied.`\n    })\n  }',
  'location': {
    'filename': 'packages/interface/src/stability.js',
    'start': {
      'line': 30,
      'column': 34
    },
    'end': {
      'line': 114,
      'column': 1
    }
  },
  'module': 'metamagical-interface/src/stability',
  'homepage': 'https://github.com/origamitower/metamagical#readme',
  'licence': 'MIT',
  'authors': ['Quildreen Motta <queen@robotlolita.me>'],
  'repository': 'git://github.com/origamitower/metamagical.git',
  'npmPackage': 'metamagical-interface',
  'copyright': '(c) 2016 Quildreen Motta',
  'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
  'stability': 'stable',
  'category': 'Stability index',
  'documentation': '\nAn index of valid stability identifiers.\n'
}), _Refinable$refine)), {
  'name': 'module stability',
  'source': 'Refinable.refine({\n  /*~\n   * Converts a textual identifier of stability to a structured\n   * representation of the stability.\n   *\n   * ---\n   * category  : Constructing stability entries\n   * stability : stable\n   *\n   * type: |\n   *   Stability.(String) => StabilityEntry\n   */\n  fromIdentifier(id) {\n    if (this.index.hasOwnProperty(id)) {\n      return this.index[id];\n    } else {\n      throw new Error(`No stability with id "${id}"`);\n    }\n  },\n\n\n  /*~\n   * An index of valid stability identifiers.\n   *\n   * ---\n   * stability : stable\n   * category  : Stability index\n   */\n  index: {\n    /*~\n     * ---\n     * stability : stable\n     * category  : Stability entry\n     */\n    deprecated: Refinable.refine({\n      index: 0,\n      name:  \'Deprecated\',\n      description: `\nThis feature is known to be problematic, and will either be entirely\nremoved from the system, or completely redesigned. You should not rely\non it.`\n    }),\n\n    /*~\n     * ---\n     * stability : stable\n     * category  : Stability entry\n     */\n    experimental: Refinable.refine({\n      index: 1,\n      name:  \'Experimental\',\n      description: `\nThis feature is experimental and likely to change (or be removed) in the\nfuture.`\n    }),\n\n    /*~\n     * ---\n     * stability : stable\n     * category  : Stability entry\n     */\n    stable: Refinable.refine({\n      index: 2,\n      name:  \'Stable\',\n      description: `\nThis feature is stable, and its API is unlikely to change (unless deemed\nnecessary for security or other important reasons). You should expect\nbackwards compatibility with the system, and a well-defined and automated\n(if possible) migration path if it changes.`\n    }),\n\n    /*~\n     * ---\n     * stability : stable\n     * category  : Stability entry\n     */\n    locked: Refinable.refine({\n      index: 3,\n      name:  \'Locked\',\n      description: `\nThis API will not change, however security and other bug fixes will still\nbe applied.`\n    })\n  }\n})',
  'location': {
    'filename': 'packages/interface/src/stability.js',
    'start': {
      'line': 30,
      'column': 0
    },
    'end': {
      'line': 114,
      'column': 3
    }
  },
  'module': 'metamagical-interface/lib/stability',
  'homepage': 'https://github.com/origamitower/metamagical#readme',
  'licence': 'MIT',
  'authors': ['Quildreen Motta <queen@robotlolita.me>'],
  'repository': 'git://github.com/origamitower/metamagical.git',
  'npmPackage': 'metamagical-interface',
  'copyright': '(c) 2016 Quildreen Motta',
  'maintainers': ['Quildreen Motta <queen@robotlolita.me> (http://robotlolita.me/)'],
  'isModule': true,
  'category': 'Metadata',
  'portability': 'portable',
  'platforms': ['ECMAScript 5', 'ECMAScript 3 (with `es5-shim`)'],
  'documentation': '\nHandles describing and normalising stability identifiers.\n'
});